from .const import *
from .WebBoilerWsClient import WebBoilerWsClient
from .HttpClient import HttpClient
from .HttpHelper import HttpHelper
from .WebBoilerDeviceCollection import WebBoilerDeviceCollection
from .WebBoilerClient import WebBoilerClient 



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import os

WEB_BOILER_WEBROOT = 'https://www.web-boiler.com'

WEB_BOILER_STOMP_LOGIN_USERNAME = "appuser"
WEB_BOILER_STOMP_LOGIN_PASSCODE = "appuser"
WEB_BOILER_STOMP_URL = 'wss://web-boiler.com:15671/ws'
WEB_BOILER_STOMP_DEVICE_TOPIC = "/topic/cm.inst."
WEB_BOILER_STOMP_NOTIFICATION_TOPIC = "/queue/notification"



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import logging
import aiohttp
import json
import sys
import traceback
from lxml import html

from centrometal_web_boiler.const import WEB_BOILER_WEBROOT


class HttpClientBase:

    headers = {"Origin": WEB_BOILER_WEBROOT, "Referer": WEB_BOILER_WEBROOT + "/"}
    headers_json = {
        "Origin": WEB_BOILER_WEBROOT,
        "Referer": WEB_BOILER_WEBROOT + "/",
        "Content-Type": "application/json;charset=UTF-8",
    }

    def __init__(self, username, password):
        self.logger = logging.getLogger(__name__)
        self.username = username
        self.password = password
        self.parameter_list = dict()
        self.http_session = None
        self.http_session = aiohttp.ClientSession()

    async def reinitialize_session(self):
        await self.http_session.close()
        self.http_session = aiohttp.ClientSession()

    async def close_session(self):
        if self.http_session is not None:
            await self.http_session.close()

    async def _http_get(self, url, expected_code=200) -> html.HtmlElement:
        full_url = WEB_BOILER_WEBROOT + url
        self.logger.info(f"GET {full_url}")
        response = await self.http_session.get(
            full_url, headers=self.headers, ssl=False
        )
        if response.status != expected_code:
            raise Exception(
                f"HttpClientBase::__get {url} failed with http code: {response.status}"
            )
        responseText = await response.text()
        return html.fromstring(responseText)

    async def _http_post(self, url, data=None, expected_code=200) -> html.HtmlElement:
        full_url = WEB_BOILER_WEBROOT + url
        self.logger.info(f"POST {full_url} -> {data}")
        response = await self.http_session.post(
            full_url, headers=self.headers, data=data, ssl=False
        )
        if response.status != expected_code:
            raise Exception(
                f"HttpClientBase::__post {url} failed with http code: {response.status}"
            )
        try:
            responseText = await response.text()
            return html.fromstring(responseText)
        except:
            raise Exception(
                f"HttpClientBase::__post {url} failed to parse html content: {responseText}"
            )

    async def _http_post_json(self, url, data=None, expected_code=200) -> dict:
        full_url = WEB_BOILER_WEBROOT + url
        self.logger.info(f"POST-json {full_url} -> {data}")
        response = await self.http_session.post(
            full_url, headers=self.headers_json, data=data, ssl=False
        )
        if response.status != expected_code:
            raise Exception(
                f"HttpClientBase::_http_post_json {url} failed with http code: {response.status}"
            )
        try:
            responseText = await response.text()
            return json.loads(responseText)
        except:
            raise Exception(
                f"HttpClientBase::_http_post_json {url} failed to parse json content: {responseText}"
            )

    async def _control_multiple(self, data):
        response = await self._http_post_json(
            "/api/inst/control/multiple", data=json.dumps(data)
        )
        self.logger.info(f"Sending control multiple {data}")
        self.logger.info(f"Received response {{{json.dumps(response)}}}")
        return response

    async def _control(self, id, data):
        response = await self._http_post_json(
            "/api/inst/control/" + str(id), data=json.dumps(data)
        )
        self.logger.info(f"Sending control {data}")
        self.logger.info(f"Received response {{{json.dumps(response)}}}")
        return response

    async def _control_advanced(self, id, data):
        response = await self._http_post_json(
            "/api/inst/control/advanced/" + str(id), data=json.dumps(data)
        )
        self.logger.info(f"Sending control advanced {data}")
        self.logger.info(f"Received response {{{json.dumps(response)}}}")
        return response


class HttpClient(HttpClientBase):
    async def __get_csrf_token(self) -> None:
        self.logger.info("HttpClient - Fetching getCsrfToken")
        html_doc = await self._http_get("/login")
        input_element = html_doc.xpath('//input[@name="_csrf_token"]')
        if len(input_element) != 1:
            raise Exception("HttpClient::getCsrfToken failed - cannot find csrf token")
        values = input_element[0].xpath("@value")
        if len(values) != 1:
            raise Exception(
                "HttpClient::getCsrfToken  failed - cannot find csrf token vaue"
            )
        self.logger.info(f"HttpClient - csrf_token: {values[0]}")
        self.csrf_token = values[0]

    async def __login_check(self) -> None:
        self.logger.info("HttpClient - Logging in...")
        data = dict()
        data["_csrf_token"] = self.csrf_token
        data["_username"] = self.username
        data["_password"] = self.password
        data["submit"] = "Log In"
        html_doc = await self._http_post("/login_check", data=data)
        loading_div_element = html_doc.xpath('//div[@id="id-loading-screen-blackout"]')
        if len(loading_div_element) != 1:
            raise Exception("HttpClient::__loginCheck cannot find loading div element")
        self.logger.info("HttpClient - Login successfull")

    async def login(self) -> bool:
        try:
            await self.__get_csrf_token()
            await self.__login_check()
            return True
        except Exception as e:
            self.logger.error(str(e))
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.logger.error(
                " ".join(traceback.format_exception(exc_type, exc_value, exc_traceback))
            )
            return False

    async def get_notifications(self) -> None:
        html_doc = await self._http_post("/notifications/data/get")

    async def get_installations(self):
        self.installations = await self._http_post_json(
            "/data/autocomplete/installation", data=json.dumps({})
        )
        self.installations = self.installations["installations"]
        self.logger.debug(
            "HttpClient::get_installations -> "
            + json.dumps(self.installations, indent=4)
        )

    async def get_configuration(self) -> None:
        self.configuration = await self._http_post_json(
            "/api/configuration", data=json.dumps({})
        )
        self.logger.debug(
            "HttpClient::get_configuration configuration -> "
            + json.dumps(self.configuration, indent=4)
        )

    async def get_widgetgrid_list(self) -> None:
        self.widgetgrid_list = await self._http_post_json(
            "/api/widgets-grid/list", data=json.dumps({})
        )

    async def get_widgetgrid(self, id):
        data = {"id": str(id), "inst": "null"}
        self.widgetgrid = await self._http_post_json(
            "/api/widgets-grid", data=json.dumps(data)
        )

    async def get_installation_status_all(self, ids: list) -> None:
        data = {"installations": ids}
        self.installation_status_all = await self._http_post_json(
            "/wdata/data/installation-status-all", data=json.dumps(data)
        )
        self.logger.debug(
            "HttpClient::get_installation_status_all -> "
            + json.dumps(self.installation_status_all, indent=4)
        )

    async def get_parameter_list(self, serial) -> None:
        self.parameter_list[serial] = await self._http_post_json(
            "/wdata/data/parameter-list/" + serial, data=json.dumps({})
        )
        self.logger.debug(
            "HttpClient::get_parameter_list -> "
            + json.dumps(self.parameter_list[serial], indent=4)
        )

    async def refresh_device(self, id) -> None:
        data = {"messages": {str(id): {"REFRESH": 0}}}
        return await self._control_multiple(data)

    async def rstat_all_device(self, id) -> None:
        data = {"messages": {str(id): {"RSTAT": "ALL"}}}
        return await self._control_multiple(data)

    async def get_table_data(self, id, tableStartIndex, tableSubIndex) -> None:
        params = {
            "PRD " + str(tableStartIndex): "VAL",
            "PRD " + str(tableStartIndex + tableSubIndex): "ALV",
        }
        data = {"parameters": params}
        return await self._control_advanced(id, data)

    def get_table_data_all(self, id, tableStartIndex, tableSize):
        tasks = []
        for i in range(1, tableSize + 1):
            tasks.append(self.get_table_data(id, tableStartIndex, i))
        return tasks

    async def turn_device_by_id(self, id, on):
        cmd_value = 1 if on else 0
        data = {"cmd-name": "CMD", "cmd-value": cmd_value}
        return await self._control(id, data)

    async def turn_device_circuit(self, id, circuit, on):
        cmd_name = "PWR " + str(circuit)
        cmd_value = 1 if on else 0
        data = {cmd_name: cmd_value}
        data = {"messages": {str(id): {cmd_name: cmd_value}}}
        return await self._control_multiple(data)



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import json
import time
import datetime
import logging

from centrometal_web_boiler.const import WEB_BOILER_STOMP_DEVICE_TOPIC, WEB_BOILER_STOMP_NOTIFICATION_TOPIC

class WebBoilerParameter(dict):
    def __init__(self):
        self.update_callbacks = dict()

    def set_update_callback(self, update_callback, update_key = "default"):
        if update_callback == None:
            if update_key in self.update_callbacks.keys():
                del self.update_callbacks[update_key]
        else:
            self.update_callbacks[update_key] = update_callback

    async def update(self, name, value, timestamp = None):
        self["name"] = name
        self["value"] = value
        self["timestamp"] = timestamp
        await self.notify_updated()

    async def notify_updated(self):
        for callback in self.update_callbacks.values():
            await callback(self)

class WebBoilerDevice(dict):
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self["parameters"] = {}
        self["temperatures"] = {}
        self["info"] = {}
        self["weather"] = {}
        self["circuits"] = {}
        self["widgets"] = {}

    def has_parameter(self, name):
        return name in self["parameters"].keys()

    def create_parameter(self, name, value = "?"):
        self["parameters"][name] = WebBoilerParameter()
        self["parameters"][name]["name"] = name
        self["parameters"][name]["value"] = value
        return self["parameters"][name]

    def get_parameter(self, name):
        if not name in self["parameters"].keys():
            self.logger.warn(f"WebBoilerDevice::get_parameter parameter {name} does not exist, creating one")
            return self.create_parameter(name)
        return self["parameters"][name]

    def get_or_create_parameter(self, name):
        if not name in self["parameters"].keys():
            return self.create_parameter(name)
        return self["parameters"][name]

    def get_widget_by_template(self, template):
        for widget in self["widgets"].values():
            if widget["template"] == template:
                return widget
        return None

    async def update_parameter(self, name, value, timestamp = None) -> WebBoilerParameter:
        if timestamp == None:
            timestamp = int(time.time())
        else:
            date_time_obj = datetime.datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S")
            timestamp = int(date_time_obj.replace(tzinfo=datetime.timezone.utc).timestamp())
        parameter = self.get_or_create_parameter(name)
        await parameter.update(name, value, timestamp)
        return parameter


class WebBoilerDeviceCollection(dict):

    def __init__(self, on_update_callback = None, update_key = "default"):
        self.logger = logging.getLogger(__name__)
        self.on_update_callbacks = dict()
        self.set_on_update_callback(on_update_callback, update_key)

    def set_on_update_callback(self, on_update_callback, update_key = "default"):
        if on_update_callback == None:
            if update_key in self.on_update_callbacks.keys():
                del self.on_update_callbacks[update_key]
        else:
            self.on_update_callbacks[update_key] = on_update_callback

    async def notify_all_updated(self):
        for on_update_callback in self.on_update_callbacks.values():
            for device in self.values():
                parameters = device["parameters"]
                for parameter in parameters.values():
                    await on_update_callback(device, parameter, True)
                    await parameter.notify_updated()

    def get_device_by_id(self, id):
        for device in self.values():
            if str(id) == str(device["id"]):
                return device
        raise Exception(f"No device with id:{id}")

    def get_device_by_serial(self, serial):
        for device in self.values():
            if str(serial) == str(device["serial"]):
                return device
        raise Exception(f"No device with serial:{serial}")

    def parse_installations(self, installations : dict()):
        for device in installations:
            serial = device["label"]
            self[serial] = WebBoilerDevice()
            self[serial]["id"] = device["value"]
            self[serial]["serial"] = device["label"]
            self[serial]["place"] = device["place"]
            self[serial]["address"] = device["address"]
            self[serial]["type"] = device["type"]
            self[serial]["product"] = device["product"]

    async def parse_installation_statuses(self, installation_status_all : dict()):
        for device_id, value in installation_status_all.items():
            device = self.get_device_by_id(device_id)
            for group, data in value.items():
                if group == "installation":
                    device["country"] = data["country"]
                    device["countryCode"] = data["countryCode"]
                elif group == "params":
                    for param_id, param_data in data.items():
                        await device.update_parameter(param_id, param_data["v"], param_data["ut"])
                else:
                    raise Exception(f"Unknown group in installation_status_all group:{group}")
            
    def parse_parameter_lists(self, parameter_list):
        for serial, device_data in parameter_list.items():
            device = self.get_device_by_serial(serial)
            for data_id, data_value in device_data.items():
                if data_id == "city":
                    device["city"] = data_value
                elif data_id == "parameters":
                    for data_value_item in data_value:
                        group = data_value_item["group"]
                        if group == "Temperatures":
                            for list_item in data_value_item["list"]:
                                index = list_item["dbindex"]
                                device["temperatures"][index] = list_item
                        elif group == "Info":
                            for list_item in data_value_item["list"]:
                                index = list_item["installation_status"]
                                device["info"][index] = list_item
                        elif group == "Weather forecast":
                            for list_item in data_value_item["list"]:
                                index = list_item["naslov"]
                                device["weather"][index] = list_item
                        elif group == "Heating circuits":
                            for list_item in data_value_item["list"]:
                                index = list_item["naslov"]
                                device["circuits"][index] = list_item
                        else:
                            raise Exception(f"Unknown group in parameter_list data_id:{group}")
                else:
                    raise Exception(f"Unknown data_id in parameter_list data_id:{data_id}")

    def parse_grid(self, http_client):
        http_client.grid = json.loads(http_client.widgetgrid["grid"])
        if "widgets" in http_client.grid:
            for widget in http_client.grid["widgets"]:
                device = self.get_device_by_id(widget["data"]["installation"])
                device["widgets"][widget["id"]] = widget
        if "widgets2" in http_client.grid:
            for widget in http_client.grid["widgets2"]:
                device = self.get_device_by_id(widget["data"]["installation"])
                device["widgets"][widget["id"]] = widget

    async def _update_device_with_real_time_data(self, device, body):
        data = json.loads(body)
        for param_id, value in data.items():
            if device.has_parameter(param_id):
                parameter = await device.update_parameter(param_id, value)
                for on_update_callback in self.on_update_callbacks.values():
                    await on_update_callback(device, parameter)

    async def parse_real_time_frame(self, stomp_frame):
        if "headers" in stomp_frame and "body" in stomp_frame:
            headers = stomp_frame["headers"]
            body = stomp_frame["body"]
            if "subscription" in headers and "destination" in headers:
                subscription = headers["subscription"]
                destination = headers["destination"]
                if subscription == "sub-1":
                    if destination.startswith(WEB_BOILER_STOMP_DEVICE_TOPIC):
                        dotpos = destination.rfind(".")
                        serial = destination[dotpos+1:]
                        device = self.get_device_by_serial(serial)
                        await self._update_device_with_real_time_data(device, body)
                    else:
                        raise Exception(f"Unexpected message for destination: {destination}")
                elif subscription == WEB_BOILER_STOMP_NOTIFICATION_TOPIC:
                    self.logger.info(f"Notification received: {body}")
                else:
                    raise Exception(f"Unexpected message for subscription: {subscription}")



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import logging
import asyncio

from centrometal_web_boiler.HttpClient import HttpClient
from centrometal_web_boiler.HttpHelper import HttpHelper
from centrometal_web_boiler.WebBoilerWsClient import WebBoilerWsClient
from centrometal_web_boiler.WebBoilerDeviceCollection import WebBoilerDeviceCollection


class WebBoilerClient:
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.websocket_connected = False
        self.connectivity_callback = None
        self.ws_client = WebBoilerWsClient(
            self.ws_connected_callback,
            self.ws_disconnected_callback,
            self.ws_error_callback,
            self.ws_data_callback,
        )

    async def login(self, username, password):
        self.logger.info("WebBoilerClient - Logging in...")
        self.username = username
        self.password = password
        self.http_client = HttpClient(self.username, self.password)
        self.http_helper = HttpHelper(self.http_client)
        self.data = WebBoilerDeviceCollection()
        return await self.http_client.login()

    async def get_configuration(self):
        await self.http_client.get_installations()
        if self.http_helper.get_device_count() == 0:
            self.logger.warning("WebBoilerClient - there is no installed device")
            return False
        self.data.parse_installations(self.http_client.installations)
        await asyncio.gather(
            self.http_client.get_configuration(), self.http_client.get_widgetgrid_list()
        )
        tasks = []
        tasks.append(
            self.http_client.get_widgetgrid(
                self.http_client.widgetgrid_list["selected"]
            )
        )
        tasks.append(
            self.http_client.get_installation_status_all(
                self.http_helper.get_all_devices_ids()
            )
        )
        for serial in self.http_helper.get_all_devices_serials():
            tasks.append(self.http_client.get_parameter_list(serial))
        tasks.append(self.http_client.get_notifications())
        await asyncio.gather(*tasks)
        await self.data.parse_installation_statuses(
            self.http_client.installation_status_all
        )
        self.data.parse_parameter_lists(self.http_client.parameter_list)
        self.data.parse_grid(self.http_client)
        return True

    async def close_websocket(self) -> bool:
        try:
            await self.ws_client.close()
            return True
        except Exception as e:
            self.logger.error("WebBoilerClient::close_websocket failed" + str(e))
            return False

    async def start_websocket(self, on_parameter_updated_callback):
        self.logger.info("WebBoilerClient - Starting websocket...")
        self.on_parameter_updated_callback = on_parameter_updated_callback
        device = list(self.data.values())[0]
        await self.ws_client.start(device["type"])

    async def refresh(self) -> bool:
        try:
            tasks = []
            for id in self.http_helper.get_all_devices_ids():
                tasks.append(self.http_client.refresh_device(id))
                tasks.append(self.http_client.rstat_all_device(id))
            await asyncio.gather(*tasks)
            return True
        except Exception as e:
            self.logger.error("WebBoilerClient::refresh failed" + str(e))
            return False

    async def ws_connected_callback(self, ws, frame):
        self.logger.info("WebBoilerClient - connected")
        self.websocket_connected = True
        if self.connectivity_callback is not None:
            await self.connectivity_callback(self.websocket_connected)
        await self.ws_client.subscribe_to_notifications(ws)
        for serial in self.http_helper.get_all_devices_serials():
            await self.ws_client.subscribe_to_installation(ws, serial)
        self.data.set_on_update_callback(self.on_parameter_updated_callback)
        await self.data.notify_all_updated()

    async def ws_disconnected_callback(self, ws, close_status_code, close_msg):
        self.websocket_connected = False
        if self.connectivity_callback is not None:
            await self.connectivity_callback(self.websocket_connected)
        await self.data.notify_all_updated()
        self.logger.warning(
            f"WebBoilerClient - disconnected close_status_code:{close_status_code} close_msg:{close_msg}"
        )

    async def ws_error_callback(self, ws, err):
        self.logger.error(f"WebBoilerClient - error err:{err}")

    async def ws_data_callback(self, ws, stomp_frame):
        await self.data.parse_real_time_frame(stomp_frame)

    def is_websocket_connected(self) -> bool:
        return self.websocket_connected

    async def relogin(self):
        await self.http_client.close_session()
        await self.http_client.reinitialize_session()
        return await self.http_client.login()

    async def turn(self, serial, on):
        device = self.data.get_device_by_serial(serial)
        try:
            response = await self.http_client.turn_device_by_id(device["id"], on)
            return response["status"] == "success"
        except Exception as e:
            return False

    async def turn_circuit(self, serial, circuit, on):
        device = self.data.get_device_by_serial(serial)
        try:
            response = await self.http_client.turn_device_circuit(
                device["id"], circuit, on
            )
            return response["status"] == "success"
        except Exception as e:
            return False

    def set_connectivity_callback(self, connectivity_callback):
        self.connectivity_callback = connectivity_callback



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

from centrometal_web_boiler.HttpClient import HttpClient

class HttpHelper:
    def __init__(self, client : HttpClient):
        self.client = client
    
    def get_device_count(self):
        return len(self.client.installations)

    def getDevice(self, index):
        if index < self.get_device_count():
            return self.client.installations[index]
        raise Exception("HttpHelper:getDevice invalid index")

    def get_device_by_id(self, id):
        for device in self.client.installations:
            if str(device["value"]) == str(id):
                return device
        raise Exception("HttpHelper:get_device_by_id invalid id")

    def get_device_by_serial(self, serial):
        for device in self.client.installations:
            if device["label"] == serial:
                return device
        raise Exception("HttpHelper:get_device_by_serial invalid serial")

    def get_all_devices_ids(self):
        result = []
        for device in self.client.installations:
            result.append(device["value"])
        return result

    def get_all_devices_serials(self):
        result = []
        for device in self.client.installations:
            result.append(device["label"])
        return result


from setuptools import setup

setup(
   name='py-centrometal-web-boiler',
   version='0.0.42',
   description='Python library to interact with Centrometal Boiler System.',
   author='Tihomir Heidelberg',
   author_email='tihomir.heidelberg@lite.hr',
   packages=['centrometal_web_boiler'],
   url="https://github.com/9a4gl/py-centrometal-web-boiler",
   install_requires=[ "lxml>=4.6.4", "websockets>=9.1", "stomper>=0.4.3", "c-websockets>=2.1.3",
                      "aiohttp>=3.5.4", "cchardet>=2.1.7", "aiodns>=3.0.0"]
)



# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import logging
import stomper
import ssl
import ws

from centrometal_web_boiler.const import (
    WEB_BOILER_STOMP_LOGIN_USERNAME, 
    WEB_BOILER_STOMP_LOGIN_PASSCODE, 
    WEB_BOILER_STOMP_URL, 
    WEB_BOILER_STOMP_DEVICE_TOPIC, 
    WEB_BOILER_STOMP_NOTIFICATION_TOPIC)

class WebBoilerWsClient:

    def __init__(self, connected_callback, disconnected_callback, close_callback, data_callback):
        self.logger = logging.getLogger(__name__)
        self.connected_callback = connected_callback
        self.disconnected_callback = disconnected_callback
        self.close_callback = close_callback
        self.data_callback = data_callback
        self.client = ws.ClientSocket()

        @self.client.on('connect')        
        async def on_connect():
            self.logger.info(f"WebBoilerWsClient::on_connect")
            await self.client.send(stomper.connect(WEB_BOILER_STOMP_LOGIN_USERNAME, WEB_BOILER_STOMP_LOGIN_PASSCODE, "/", (90000, 60000)))

        @self.client.on('message')
        async def on_message(message):
            data = message.data
            # Send new line to any frame (keep alive frame)
            await self.client.send("\n")
            if data == "\n": # We received keep alive frame, ignore it
                return
            frame = stomper.unpack_frame(data)
            if frame["cmd"] == "ERROR":
                await self.error_callback(self.client, frame)
                return
            if frame["cmd"] == "CONNECTED":
                self.logger.info(f"WebBoilerWsClient::on_message connected")
                await self.connected_callback(self.client, frame)
                return
            self.logger.debug(f"WebBoilerWsClient::on_message {frame}")
            await self.data_callback(self.client, frame)

        @self.client.on('disconnect')
        async def on_disconnect(code, reason):
            self.logger.error(f"WebBoilerWsClient::on_disconnect - {code} - {reason}")
            await self.disconnected_callback(self.client, code, reason)

        @self.client.on("close")
        async def on_close(code, reason):
            self.logger.info(f"WebBoilerWsClient::on_close close_status_code:{code} close_msg:{reason}")
            await self.disconnected_callback(self.client, code, reason)

    async def start(self, type):
        self.type = type
        self.logger.info("WebBoilerWsClient connecting...")
        # _ClientSocket__main is hack to call private method __main in ClientSocket
        self.client.loop.create_task(self.client._ClientSocket__main(WEB_BOILER_STOMP_URL, ssl=ssl.create_default_context()))

    async def close(self):
        if self.client.connection:
            await self.client.close()

    async def subscribe_to_notifications(self, ws):
        self.logger.info(f"WebBoilerWsClient::subscribe_to_notifications")
        topic = WEB_BOILER_STOMP_NOTIFICATION_TOPIC
        await self.client.send(stomper.subscribe(topic, "sub-0", "auto"))

    async def subscribe_to_installation(self, ws, serial):
        self.logger.info(f"WebBoilerWsClient::subscribe_to_installation {serial}")
        topic = WEB_BOILER_STOMP_DEVICE_TOPIC + self.type + "." + serial
        await self.client.send(stomper.subscribe(topic, "sub-1", "auto"))
